/***************************************************************************
 *                                                                         *
 *                            LibQNotify                                   *
 *                                                                         *
 *            Copyright (C) 2006 Holger Macht <holger@homac.de>            *
 *                                                                         *
 *                 Author: Holger Macht <holger@homac.de>                  *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify it *
 * under the terms of the GNU General Public License as published by the   *
 * Free Software Foundation; either version 2 of the License, or (at you   *
 * option) any later version.                                              *
 *                                                                         *
 * This program is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
 * General Public License for more details.                                *
 *                                                                         *
 * You should have received a copy of the GNU General Public License along *
 * with this program; if not, write to the Free Software Foundation, Inc., *
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA                  *
 *                                                                         *
 ***************************************************************************/

#include "qnotify_container.h"

#include <qsize.h>
#include <qpoint.h>
#include <qapplication.h>

QNotifyContainer::QNotifyContainer(QNotifyContainer * parent, const char *name, WFlags f)
	: QWidget(parent, name, f)
{
	_rounding_factor = 3000;
}

QNotifyContainer::~QNotifyContainer()
{
}

QSize QNotifyContainer::desktopGeometry()
{
	QSize s(qApp->desktop()->width(), qApp->desktop()->height());
	return s;
}

QPoint QNotifyContainer::centerPosition(int w1, int h1, int w2, int h2)
{
	int x, y;

	x = w1 / 2 - w2 / 2;
	y = h1 / 2 - h2 / 2;

	return QPoint(x, y);
}

bool QNotifyContainer::event(QEvent * e)
{
	switch (e->type()) {
	case QEvent::Paint:
		bitBlt(this, 0, 0, &_buffer);
		return TRUE;
	default:
		return QWidget::event(e);
	}
}

void QNotifyContainer::setRounding(int factor)
{
	_rounding_factor = factor;
}

void QNotifyContainer::setOwnMask(int w, int h)
{
	initMask(QSize(w, h));

	if (_rounding_factor >= 0)
		_maskPainter.drawRoundRect(0, 0, w, h, _rounding_factor / w,
					   _rounding_factor / h);
	else
		_maskPainter.drawRect(0, 0, w, h);

	setMask(_mask);

	_maskPainter.end();
}

void QNotifyContainer::drawBackground(QRect rect)
{
	if (_rounding_factor >= 0) {
		_bufferPainter.drawRoundRect(rect, _rounding_factor / rect.width(),
					     _rounding_factor / rect.height());
	} else {

		_bufferPainter.drawRect(rect);
	}
}

void QNotifyContainer::initMask(QSize size)
{
	_mask.resize(size.width(), size.height());
	_mask.fill(Qt::color0);

	_maskPainter.begin(&_mask);
	_maskPainter.setBackgroundMode(Qt::TransparentMode);
	_maskPainter.setPen(Qt::color0);
	_maskPainter.setBrush(Qt::color1);
}

void QNotifyContainer::initBuffer(QSize size)
{
	_buffer.resize(size.width(), size.height());

	if (_bufferPainter.isActive()) {
		_bufferPainter.end();
	}

	_bufferPainter.begin(&_buffer);

	_bufferPainter.setBackgroundMode(Qt::TransparentMode);
	_bufferPainter.setPen(Qt::black);
	_bufferPainter.setBrush(backgroundColor());

}

void QNotifyContainer::exitTrue()
{
	qApp->exit(0);
}

void QNotifyContainer::exitFalse()
{
	qApp->exit(1);
}
