/***************************************************************************
 *                                                                         *
 *                            LibQNotify                                   *
 *                                                                         *
 *            Copyright (C) 2006 Holger Macht <holger@homac.de>            *
 *                                                                         *
 *                 Author: Holger Macht <holger@homac.de>                  *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify it *
 * under the terms of the GNU General Public License as published by the   *
 * Free Software Foundation; either version 2 of the License, or (at you   *
 * option) any later version.                                              *
 *                                                                         *
 * This program is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
 * General Public License for more details.                                *
 *                                                                         *
 * You should have received a copy of the GNU General Public License along *
 * with this program; if not, write to the Free Software Foundation, Inc., *
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA                  *
 *                                                                         *
 ***************************************************************************/

#include "qnotify_label.h"

#include <qrect.h>
#include <qsimplerichtext.h>
#include <qapplication.h>

#include <iostream>

QNotifyLabel::QNotifyLabel(QNotifyContainer *parent, const char *name, WFlags f)
	: QNotifyContainer(parent, name, f)
{
	_my_font.setFamily("Arial");
	_my_font.setPointSize(12);

	_margin = 0;
}

QNotifyLabel::~QNotifyLabel()
{
	delete _srt;
}

void QNotifyLabel::setText(const QString &text, int fontsize)
{
	_text = text;

	if (fontsize < 0) {
		_my_font.setPointSize(_my_font.pointSize());
	}
	else {
		_my_font.setPointSize(fontsize);
	}
}

QString QNotifyLabel::text()
{
	return _text;
}

QSize QNotifyLabel::init(QSize availableSize, bool transparent, bool shadow)
{
	_text.replace("\\n", "\n");

	int pos;

	if (_text.find("align=", 0, false) == -1) {
		_text.prepend("<center>");
	}

	if (transparent)
		_text.prepend("<b>");

	_srt = new QSimpleRichText(tr(_text), _my_font);

	QRect r(0, 0, availableSize.width(), availableSize.height());

	if (availableSize.width() < 0) {
		_srt->setWidth(desktopGeometry().width() - desktopGeometry().width() / 12);
		r.setWidth(_srt->widthUsed() + 2 * _margin);
	} else
		_srt->setWidth(availableSize.width());

	if (availableSize.height() < 0) {
		r.setHeight(_srt->height() + 2 * _margin);
	}

	_srt->setWidth(r.width() - 2 * _margin);

	if (_srt->height() <= r.width())
		_srt_position =
		    centerPosition(r.width(), r.height(), _srt->widthUsed(), _srt->height());
	else
		_srt_position = QPoint(_margin, _margin);

	initBuffer(r.size());
	drawBackground(r);
	setOwnMask(r.width(), r.height());

	QColorGroup g(colorGroup());

	//draw the shadow
	if (shadow) {
		g.setColor(QColorGroup::Text, backgroundColor().dark(175));
		_srt->draw(&_bufferPainter, _margin + 3, _srt_position.y(), r, g, 0);
	}

	//draw the text
	g.setColor(QColorGroup::Text, foregroundColor());
	_srt->draw(&_bufferPainter, _margin, _srt_position.y(), r, g, 0);

	_bufferPainter.end();

	_srt->setWidth(_srt->widthUsed());

	resize(r.width(), r.height());
	update();

	return QSize(r.width(), r.height());
}

void QNotifyLabel::drawMask(QPainter *painter)
{
	initMask(QSize(width(), height()));

	QRect r(0, 0, width(), height());

	QColorGroup g(colorGroup());
	g.setColor(QColorGroup::Text, foregroundColor());

	_srt->draw(painter, _margin, _srt_position.y(), r, g, 0);

	painter->end();
	setMask(_mask);
}
