/***************************************************************************
 *                                                                         *
 *                            LibQNotify                                   *
 *                                                                         *
 *            Copyright (C) 2006 Holger Macht <holger@homac.de>            *
 *                                                                         *
 *                 Author: Holger Macht <holger@homac.de>                  *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify it *
 * under the terms of the GNU General Public License as published by the   *
 * Free Software Foundation; either version 2 of the License, or (at you   *
 * option) any later version.                                              *
 *                                                                         *
 * This program is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
 * General Public License for more details.                                *
 *                                                                         *
 * You should have received a copy of the GNU General Public License along *
 * with this program; if not, write to the Free Software Foundation, Inc., *
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA                  *
 *                                                                         *
 ***************************************************************************/

#ifndef QNOTIFY_LABEL_H
#define QNOTIFY_LABEL_H

#include <qstring.h>

#include "qnotify_container.h"

class QSimpleRichText;
class QFont;

class QNotifyLabel : public QNotifyContainer {
//Q_OBJECT

public:
	/** @brief constructor */
	QNotifyLabel(QNotifyContainer *parent, const char *name = 0, WFlags f = 0);
	
	/** @brief destructor */
	~QNotifyLabel();
	
	/** @brief set the text with a fontsize
	 *
	 * @param text the text to set
	 * @param fontsize the fontsize to st
	 */
	void setText(const QString &text, int fontsize = -1);

	/** @brief init the label
	 *
	 * @param availableSize the size the label gets to draw itself
	 * @param transparent if the text should be transparent
	 * @param shadow if a shadow should be drawn for the text
	 *
	 * @return the actual size the label needs
	 */
	QSize init(QSize availableSize, bool transparent = false, bool shadow = true);

	/** @brief draws the mask
	 *
	 * @param painter the painter to draw on
	 */
	void drawMask(QPainter *painter);

	/** @brief get the text shown */
	QString text();

private:
	/** @brief object holding the text */
	QSimpleRichText *_srt;

	/** @brief the text to be displayed */
	QString _text;

	/** @brief the position of the QSimpleRichText */
	QPoint _srt_position;
	
	/** @brief the margin between the text and the border */
	int _margin;

	/** @brief the font to be used to draw the text */
	QFont _my_font;
};

#endif // QNOTIFY_LABEL_H
