/***************************************************************************
 *                                                                         *
 *                            LibQNotify                                   *
 *                                                                         *
 *            Copyright (C) 2006 Holger Macht <holger@homac.de>            *
 *                                                                         *
 *                 Author: Holger Macht <holger@homac.de>                  *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify it *
 * under the terms of the GNU General Public License as published by the   *
 * Free Software Foundation; either version 2 of the License, or (at you   *
 * option) any later version.                                              *
 *                                                                         *
 * This program is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
 * General Public License for more details.                                *
 *                                                                         *
 * You should have received a copy of the GNU General Public License along *
 * with this program; if not, write to the Free Software Foundation, Inc., *
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA                  *
 *                                                                         *
 ***************************************************************************/

#include "qnotify_popup.h"
#include "qnotify_label.h"
#include "qnotify_progressbar.h"

#include <qapplication.h>
#include <qtimer.h>

QNotifyPopup::QNotifyPopup(QNotifyPopup *parent, const char *name, WFlags f)
	: QNotifyContainer(parent, name, f)
{
	_show_time = -1;
	_hide_time = 1000;
	_times_shown = 1;
	_transparent = false;
	_shadow = true;
	_margin = 10;
	_default_position = QNOTIFY_POS_UNDEFINED;
	_rounding_factor = 3000;
	_progress_bar = NULL;

	setFocusPolicy(NoFocus);
//	setBackgroundMode( NoBackground );

 	setPaletteBackgroundColor("#3b508a");
	setPaletteForegroundColor("white");

	showtimer = new QTimer(this, "showtimer");
	hidetimer = new QTimer(this, "hidetimer");

	connect(showtimer, SIGNAL(timeout()), this, SLOT(hidePopup()));
	connect(hidetimer, SIGNAL(timeout()), this, SLOT(popup()));
}

QNotifyPopup::~QNotifyPopup()
{
}

void QNotifyPopup::popup(const QString &text)
{
	QNotifyPopup *p = new QNotifyPopup(0);

	p->addTextWidget(text);
	p->setPosition(QNOTIFY_POS_BOTTOM_LEFT);
	p->render();
	p->popup();
}

void QNotifyPopup::setTransparent()
{
	_transparent = true;
}

void QNotifyPopup::setTimesShown(int times_shown)
{
	_times_shown = times_shown;
}

void QNotifyPopup::setShowInterval(int show_time)
{
	_show_time = show_time;
}

void QNotifyPopup::setHideInterval(int hide_time)
{
	_hide_time = hide_time;
}

void QNotifyPopup::setText(const QString &text, int widget_num)
{
	int num = 0;
	for (QValueList< QWidget* >::iterator it = _widgets.begin();
	     it != _widgets.end(); it++) {
		
		QNotifyLabel *l = dynamic_cast< QNotifyLabel* > (*it);
		if( l != NULL) {

			if (num == widget_num) {
				l->setText(text);
			}
			num++;
		}

	}
}

void QNotifyPopup::setBackgroundColor(const QString &color)
{
	setPaletteBackgroundColor(color);
}

void QNotifyPopup::setTextColor(const QString &color)
{
	setPaletteForegroundColor(color);
}

void QNotifyPopup::setPosition(QNOTIFY_POSITION pos)
{
	_default_position = pos;
}

void QNotifyPopup::addTextWidget(const QString &text, int fontsize, int align)
{
	QNotifyLabel *l = new QNotifyLabel(this);
	l->setText(text, fontsize);
	addWidget(l, align);
}

void QNotifyPopup::addProgressBar(int height, const QString &color)
{
	_progress_bar = new QNotifyProgressWidget(this);
	_progress_bar->setColor(color);
	_progress_bar->setHeight(height);
	addWidget(_progress_bar, Qt::AlignHCenter);
}

void QNotifyPopup::setProgress(int progress, const QString &text)
{
	if (_progress_bar == NULL) {
		return;
	}

	_progress_bar->setProgress(progress, text);
}

void QNotifyPopup::setMargin(int margin)
{
	_margin = margin;
}

void QNotifyPopup::addWidget(QWidget *widget, int align)
{
	_widgets.append(widget);
	_align.append(align);
}

void QNotifyPopup::disableShadow()
{
	_shadow = false;
}

void QNotifyPopup::setRounding(int factor)
{
	QNotifyContainer::setRounding(factor);
	_rounding_factor = factor;
}

void QNotifyPopup::render(int w, int h) {
	if (_widgets.size() == 0) {
		addTextWidget("Notification", 12);
	}

	QSize total_size(0, 0);
	for (QValueList< QWidget* >::iterator it = _widgets.begin();
	     it != _widgets.end(); it++) {
		
		QNotifyLabel *l = dynamic_cast< QNotifyLabel* > (*it);
		if( l != NULL) {
			l->setRounding(_rounding_factor);
			l->init(QSize(w - 2 * _margin, h - 2 * _margin), _transparent, _shadow);

			if (_transparent) {
				initMask(QSize(l->width(), l->height()));
				l->drawMask(&_maskPainter);
				setMask(_mask);
			}
			
			// calculate the time shown if no value was given
			if (_show_time < 0)
				_show_time = (int)((float)l->text().length() * 90 + 2000);

		}
		
		QNotifyProgressWidget *p = dynamic_cast< QNotifyProgressWidget* > (*it);
		if( p != NULL) {
			p->init(total_size.width());
//			p->init(w - 2 * _margin);
		}


		if ((*it)->width() > total_size.width()) {
			total_size.setWidth((*it)->width());
		}

		total_size.setHeight(total_size.height() + (*it)->height());

		(*it)->show();
			
	}
	total_size.setWidth(total_size.width() + 2 * _margin);
	total_size.setHeight(total_size.height() + 2 * _margin);

	resize(total_size.width(), total_size.height() );

	initBuffer(total_size);
	drawBackground(QRect(0, 0, total_size.width(), total_size.height()));

	if (!_transparent)
		setOwnMask(total_size.width(), total_size.height());

	update();

	_bufferPainter.end();

	int sum_height = _margin;

	QValueList< int >::iterator ti = _align.begin();
 	for (QValueList< QWidget* >::iterator it = _widgets.begin();
	     it != _widgets.end(); it++, ++ti) {

		int align;

		switch (*ti) {
		case Qt::AlignLeft:
			align = _margin;
			break;
		case Qt::AlignHCenter:
			align = width() / 2 - (*it)->width() / 2;
			break;
		case Qt::AlignRight:
			align = width() - _margin - (*it)->width();
			break;
		}

		(*it)->move(align, sum_height);
		sum_height += (*it)->height();
	}

	if (_default_position != QNOTIFY_POS_UNDEFINED)
		setDefaultPosition(_default_position);

}

bool QNotifyPopup::setSliding(QValueList< int > x_positions,
			      QValueList< int > y_positions,
			      QValueList< int > intervals)
{
	_x_positions = x_positions;
	_y_positions = y_positions;
	_intervals = intervals;

	initSliding();

	return true;
}

void QNotifyPopup::initSliding()
{
	move(_x_positions[0], _y_positions[0]);

	slideTimer = new QTimer(this);
	connect(slideTimer, SIGNAL(timeout()), this, SLOT(slide()));
	slideTimer->start(_intervals[0], true);
}

void QNotifyPopup::slide()
{
	static QValueList < int >::iterator xIt = _x_positions.begin();
	static QValueList < int >::iterator yIt = _y_positions.begin();
	static QValueList < int >::iterator ilt = _intervals.begin();

	if (xIt != _x_positions.end()) {
		move(*xIt, *yIt);
		slideTimer->start(*ilt, true);
	} else {
		slideTimer->stop();
 		emit popupFinished();
		return;
	}

	xIt++;
	yIt++;
	ilt++;
}


float QNotifyPopup::timeout()
{
	float timeout = ((float)_show_time / 1000) * (float)_times_shown;
	return timeout;
}

int QNotifyPopup::getHeight()
{
	int height = 2 * _margin;
	for (QValueList< QWidget* >::iterator it = _widgets.begin();
	     it != _widgets.end(); it++) {
		
		height += (*it)->height();
	}	
	
	return height;
}

void QNotifyPopup::setDefaultPosition(QNOTIFY_POSITION pos)
{
	int x, y, width, height;
	width = qApp->desktop()->width();
	height = qApp->desktop()->height();

	switch (pos) {
	case QNOTIFY_POS_TOP_LEFT:
		x = width / 12;
		y = (height) / 20;
		break;
	case QNOTIFY_POS_TOP_MID:
		x = width / 2 - this->width() / 2;
		y = (height) / 20;
		break;
	case QNOTIFY_POS_TOP_RIGHT:
		x = width - this->width() / 2 - width / 12;
		y = (height) / 20;

		if (x + this->width() > width - 10) {
			x = width - this->width() - 10;
		}
		break;
	case QNOTIFY_POS_MID_LEFT:
		x = width / 12;
		y = height / 2 - getHeight() / 2;
		break;
	case QNOTIFY_POS_MID_MID:
		x = width / 2 - this->width() / 2;
		y = height / 2 - getHeight() / 2;
		break;
	case QNOTIFY_POS_MID_RIGHT:
		x = width - this->width() / 2 - width / 12;
		y = height / 2 - getHeight() / 2;

		if (x + this->width() > width - 10) {
			x = width - this->width() - 10;
		}
		break;
	case QNOTIFY_POS_BOTTOM_LEFT:
		x = width / 12;
		y = height - height / 10;

//		printf("y: %d, height: %d, h-10: %d\n", this->y(), getHeight(), h-10);
		if (y + getHeight() > height - 10) {
			y = height - getHeight() - 10;
		}
		break;
	case QNOTIFY_POS_BOTTOM_MID:
		x = width / 2 - this->width() / 2;
		y = height - height / 10;

		if (y + getHeight() > height - 10) {
			y = height - getHeight() - 10;
		}
		break;
	case QNOTIFY_POS_BOTTOM_RIGHT:
		x = width - this->width() / 2 - width / 9;
		y = height - height / 9;

		if (x + this->width() > width - 10) {
			x = width - this->width() - 10;
		}

		if (y + getHeight() > height - 10) {
			y = height - getHeight() - 10;
		}
		break;
	default:
		break;
	}

	move(x, y);
}

void QNotifyPopup::popup()
{
	if (_x_positions.size() != 0) {
		show();
		slide();
		return;
	}

	show();
	if (_show_time < 0)
		_show_time = 3000;

	showtimer->start(int (_show_time), TRUE);
}

void QNotifyPopup::hidePopup()
{
	static int counter = 0;

	showtimer->stop();
	hidetimer->stop();

	counter++;

	//if that is not enough start the timer again
	if (_times_shown > counter) {
		hide();
		hidetimer->start(int (_hide_time), TRUE);
	} else {
		emit popupFinished();
	}
}

void QNotifyPopup::mousePressEvent(QMouseEvent * event)
{
	switch (event->button()) {
	case Qt::LeftButton:
		emit popupFinished();
		break;
	case Qt::RightButton:
		emit popupFinished();
		break;
	default:
		break;
	}
}

