/***************************************************************************
 *                                                                         *
 *                            LibQNotify                                   *
 *                                                                         *
 *            Copyright (C) 2006 Holger Macht <holger@homac.de>            *
 *                                                                         *
 *                 Author: Holger Macht <holger@homac.de>                  *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify it *
 * under the terms of the GNU General Public License as published by the   *
 * Free Software Foundation; either version 2 of the License, or (at you   *
 * option) any later version.                                              *
 *                                                                         *
 * This program is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
 * General Public License for more details.                                *
 *                                                                         *
 * You should have received a copy of the GNU General Public License along *
 * with this program; if not, write to the Free Software Foundation, Inc., *
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA                  *
 *                                                                         *
 ***************************************************************************/

#include "qnotify_progressbar.h"

#include <qprogressbar.h>

QNotifyProgressBar::QNotifyProgressBar(QWidget * parent, const char *name, WFlags f)
	: QProgressBar(parent, name, f)
{
}

bool QNotifyProgressBar::setIndicator ( QString & indicator, int progress, int totalSteps )
{
	static QString old_value;

	if (_text != "")
		indicator = _text;
	else {
		indicator = tr("%1 %").arg(progress);
	}
	
	if(old_value == indicator) {
		return false;
	}

	old_value = indicator;

	return true;
}

void QNotifyProgressBar::setText(const QString &text)
{
	_text = text;
}

QNotifyProgressWidget::QNotifyProgressWidget(QNotifyContainer * parent, const char *name, WFlags f)
	:QNotifyContainer(parent, name, f)
{
	_margin = 10;
	_height = -1;
	_color = "#3b508a";
	bar = new QNotifyProgressBar(this);
}

void QNotifyProgressWidget::setColor(const QString &color)
{
	_color = color;
}

void QNotifyProgressWidget::setHeight(int height)
{
	_height = height;
}

void QNotifyProgressWidget::init(int w)
{
//	QFont myfont;
//	myfont.setFamily("Arial");
//	myfont.setPointSize(5);

//	bar->setFont(myfont);

	QSize barSize(w - 2 * _margin, _height);

	if (barSize.width() < 0)
		barSize.setWidth(bar->sizeHint().width() - 2 * _margin);

	if (barSize.height() < 0)
		barSize.setHeight(bar->sizeHint().height());

	QPalette pal = bar->palette();
	
	// Set the color of the bar
	pal.setColor(QColorGroup::Highlight, QColor(_color));
	
	bar->setPalette(pal);
	
	bar->setPaletteForegroundColor(yellow);

	bar->resize(barSize);

	bar->move(_margin, _margin);
	bar->show();

	QSize s(bar->width() + 2 * _margin, bar->height() + 2 * _margin);

	initBuffer(s);
	drawBackground(QRect(QPoint(0, 0), s));
	setOwnMask(s.width(), s.height());
	resize(s);

	bar->setProgress(0);

	_bufferPainter.end();
}

void QNotifyProgressWidget::setProgress(int progress, const QString &text)
{
	if (bar == NULL)
		return;
	bar->setText(text);
	bar->setProgress(progress);
}
