/***************************************************************************
 *                                                                         *
 *                            LibQNotify                                   *
 *                                                                         *
 *            Copyright (C) 2006 Holger Macht <holger@homac.de>            *
 *                                                                         *
 *                 Author: Holger Macht <holger@homac.de>                  *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify it *
 * under the terms of the GNU General Public License as published by the   *
 * Free Software Foundation; either version 2 of the License, or (at you   *
 * option) any later version.                                              *
 *                                                                         *
 * This program is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
 * General Public License for more details.                                *
 *                                                                         *
 * You should have received a copy of the GNU General Public License along *
 * with this program; if not, write to the Free Software Foundation, Inc., *
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA                  *
 *                                                                         *
 ***************************************************************************/

#ifndef QNOTIFY_PROGRESSBAR_H
#define QNOTIFY_PROGRESSBAR_H

#include "qnotify_container.h"

#include <qprogressbar.h>

class QTimer;
class QSocketNotifier;

/** @brief custom progress bar that can show variable text */
class QNotifyProgressBar : public QProgressBar {

public:
	/** @brief constructor */
	QNotifyProgressBar(QWidget * parent, const char *name = 0, WFlags f = 0);

	/** @brief function to set the indicator text */
	virtual bool setIndicator ( QString & indicator, int progress, int totalSteps );

	/** @brief set the text the progress bar shall display
	 *
	 * @param text the text to display
	 */
	void setText(const QString &text);

private:
	/** @brief the text displayed */
	QString _text;
};

/** @brief Custom progress bar widget */
class QNotifyProgressWidget : public QNotifyContainer {

public:
	/** @brief constructor */
	QNotifyProgressWidget(QNotifyContainer * parent, const char *name = 0, WFlags f = 0);
	
	/** @brief inits the bar
	 *
	 * @param width the width the bar should habe
	 */
	void init(int width);

	/** @brief set the color of the bar
	 *
	 * @param color a Qt color
	 */
	void setColor(const QString &color);

	/** @brief set the current progress and text
	 *
	 * @param progress the current progress from 0 up to 100
	 * @param text the text to display
	 */
	void setProgress(int progress, const QString &text);

	/** @brief set the height of the bar
	 *
	 * @param height height of the bar
	 */
	void setHeight(int height);

private:
	/** @brief @ref QNotifyProgressBar object */
	QNotifyProgressBar *bar;

	/** @brief margin around the bar */
	int _margin;

	/** @brief color of the bar */
	QString _color;

	/** @brief height of the bar */
	int _height;
};

#endif // QNOTIFY_PROGRESSBAR_H
