#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
/***************************************************************************
 *                                                                         *
 *                              QNotify                                    *
 *                                                                         *
 *            Copyright (C) 2006 Holger Macht <holger@homac.de>            *
 *                                                                         *
 *                 Author: Holger Macht <holger@homac.de>                  *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify it *
 * under the terms of the GNU General Public License as published by the   *
 * Free Software Foundation; either version 2 of the License, or (at you   *
 * option) any later version.                                              *
 *                                                                         *
 * This program is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
 * General Public License for more details.                                *
 *                                                                         *
 * You should have received a copy of the GNU General Public License along *
 * with this program; if not, write to the Free Software Foundation, Inc., *
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA                  *
 *                                                                         *
 ***************************************************************************/

#include "barObj.h"
#include "settings.h"

#include <qprogressbar.h>
#include <qtimer.h>
#include <qsocketnotifier.h>
#include <qapplication.h>
#include "../libqnotify/qnotify_popup.h"

#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <iostream>

extern globopts gopts;


barObj::barObj(QNotifyPopup *popup)
{
	_popup = popup;
	pTimer = NULL;
}

barObj::~barObj()
{
}

void barObj::init()
{
	if (gopts.step != 0 && gopts.fulltime != 0) {
		pTimer = new QTimer(0);
		QObject::connect(pTimer, SIGNAL(timeout()), this, SLOT(progress()));
		pTimer->start((gopts.fulltime * 1000) / (100 / gopts.step), FALSE);
	}

	int fd = openFifo();
	if (fd < 0)
		std::cerr << "Unable to create fifo\n";
	else {
		socknoty = new QSocketNotifier(fd, QSocketNotifier::Read, 0);
		connect(socknoty, SIGNAL(activated(int)), this, SLOT(progress(int)));
	}
}

void barObj::progress()
{
	static int cur_progress = gopts.step;

	_popup->setProgress(cur_progress);

	cur_progress += gopts.step;

	if (cur_progress > 100) {
		pTimer->stop();
		delete pTimer;
//              myHide();
		qApp->exit(0);
	}
}

int barObj::openFifo()
{
	unlink(gopts.fifofile.ascii());

	int r = mkfifo(gopts.fifofile.ascii(), O_CREAT | O_NONBLOCK | S_IRWXU);
	if (r < 0) {
		std::cerr << "mkfifo: " << strerror(errno) << "\n";
		return 0;
	}

	int fd = open(gopts.fifofile.ascii(), O_RDWR | O_NONBLOCK);

	if (fd < 0) {
		std::cerr << "open fifo: " << strerror(errno) << "\n";
		return 0;
	}

	return fd;
}

char *barObj::readLine(int fd)
{
	char *buf = new(char);
	int buflen = 64;
	int max_buflen = 1024;
	int i = 0;
	int r;
	int searching = 1;

	while (searching) {
		buf = (char *)realloc(buf, buflen);
		if (!buf) {
			return NULL;
		}
		memset(buf + i, 0, buflen - i);

		while (i < buflen) {
			r = read(fd, buf + i, 1);
			if (r < 0 && errno != EINTR) {
				/* we should do something with the data */
				return NULL;
			} else if (r == 1) {
				if (buf[i] == '\n') {
					searching = 0;
					buf[i] = '\0';
					if (i == 0)
						return NULL;
					break;
				}
				i++;
			}
		}
		if (buflen >= max_buflen) {
			break;
		}
		buflen *= 2;
	}

	return buf;

}

void barObj::progress(int fd)
{
	socknoty->setEnabled(false);

	char *h = new(char);
	h = readLine(fd);
	if (h == NULL) {
		std::cerr << "Couln not read from the fifo\n";
		return;
	}

	QString line(h);
	QString text;

	int cur_prog = line.section('|', 0, 0).toInt();

	if (pTimer != NULL && pTimer->isActive() && cur_prog == 100) {
		pTimer->changeInterval((gopts.fulltime * 2) / (100 / gopts.step));
	} else {
		_popup->setProgress(cur_prog, line.section('|', 1, 1));
	}

	socknoty->setEnabled(true);

	if (cur_prog > 100) {
		qApp->exit(0);
	}

	delete h;
}
