/***************************************************************************
 *                                                                         *
 *                              QNotify                                    *
 *                                                                         *
 *            Copyright (C) 2008 Holger Macht <holger@homac.de>            *
 *                                                                         *
 *                 Author: Holger Macht <holger@homac.de>                  *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify it *
 * under the terms of the GNU General Public License as published by the   *
 * Free Software Foundation; either version 2 of the License, or (at you   *
 * option) any later version.                                              *
 *                                                                         *
 * This program is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
 * General Public License for more details.                                *
 *                                                                         *
 * You should have received a copy of the GNU General Public License along *
 * with this program; if not, write to the Free Software Foundation, Inc., *
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA                  *
 *                                                                         *
 ***************************************************************************/

#include <iostream>

#include <string.h>
#include <qtextstream.h>

#include "fileIO.h"

#include <sys/file.h>
#include <sys/time.h>

FileIO::FileIO()
{
}

int FileIO::getInactiveWidgets()
{
	return _inactiveWidgets;
}

void FileIO::setInactiveWidgets(int count)
{
	_inactiveWidgets = count;
}

#include "qnotify.h"
void FileIO::checkTimeouts()
{
	int i = 0;

	for (QValueList < wNode >::iterator it = _list.begin(); it != _list.end(); ++it) {

		int id = (*it).id;
		
		if ((*it).time < QNotify::getTime())
			i++;
	}

	setInactiveWidgets(i);
}

bool FileIO::createFile(QString filename, int flags)
{
	_file.setName(filename);

	if (!_file.open(flags)) {
		std::cerr << "error creating file" << filename << std::endl;
		return false;
	}

	if( flock(_file.handle(), LOCK_EX) ) {
		_file.close();
		return false;
	}
	return true;
}

#include <stdlib.h>
//parses buf so and overrides node
int FileIO::parseLine(wNode * node, QString buf)
{
	char *help = new(char);
	int h = 0;
	int i = 0;
//	std::cerr << "Line to parse: " << buf << "\n";


	QStringList list = QStringList::split("|", buf);

	node->x  = atoi(list[1].ascii());
	node->y  = atoi(list[2].ascii());
	node->width = atoi(list[3].ascii());
	node->height = atoi(list[4].ascii());
	node->id = atoi(list[5].ascii());
	node->time = atof(list[6].ascii());

	return 1;

}


bool FileIO::init(QString filename)
{
	createFile(filename, IO_ReadWrite | IO_Append);
	_file.reset();
	QTextStream stream( &_file );
        QString line;
        int i = 1;

        while (!stream.atEnd()) {
		wNode node;
		line = stream.readLine(); // line of text excluding '\n'
		parseLine(&node, line);
		_list.append(node);
        }

	checkTimeouts();

	if (_list.count() > 0 && _list.count() == getInactiveWidgets()) {
		_file.close();
		createFile(filename, IO_ReadWrite | IO_Truncate);
		_list.clear();
	}

	return true;

}

int FileIO::getFirst(wNode * node)
{
	if (_list.isEmpty())
		return false;
	memcpy(node, &*_list.begin(), sizeof(struct wNode));
	return 1;
}

int FileIO::getLast(wNode * node)
{
	if (_list.isEmpty())
		return false;
	memcpy(node, &_list.last(), sizeof(struct wNode));
	return 1;
}

#include <errno.h>
#include <stdlib.h>
//writes one item to the daemon socket and writes the return node (id) from the daemon to the node
int FileIO::writeFile(wNode * node)
{
	QTextStream stream(&_file);

	char *h = new(char[200]);
	int i = -1;

	i = sprintf(h, "%s|%d|%d|%d|%d|%d|%f|\n", "dummy", node->x, node->y, node->width,
		    node->height, node->id, node->time);
	stream << h;
		
	free(h);
	_file.flush();

	return 1;


}

int FileIO::close()
{
	_file.flush();
	flock(_file.handle(), LOCK_UN);
	_file.close();
	return 1;
}
