/***************************************************************************
 *                                                                         *
 *                              QNotify                                    *
 *                                                                         *
 *            Copyright (C) 2006 Holger Macht <holger@homac.de>            *
 *                                                                         *
 *                 Author: Holger Macht <holger@homac.de>                  *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify it *
 * under the terms of the GNU General Public License as published by the   *
 * Free Software Foundation; either version 2 of the License, or (at you   *
 * option) any later version.                                              *
 *                                                                         *
 * This program is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
 * General Public License for more details.                                *
 *                                                                         *
 * You should have received a copy of the GNU General Public License along *
 * with this program; if not, write to the Free Software Foundation, Inc., *
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA                  *
 *                                                                         *
 ***************************************************************************/

#include "fileManager.h"
#include "config.h"
#include <qdir.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qapplication.h>
#include <stdlib.h>

#include <iostream>

using namespace std;

fileManager::fileManager()
{
	std::string home = getenv("HOME");
	std::string homedir = home + "/.qnotify/";

	QDir d(SCRIPT_DIR);
	QDir q("homedir");

	searchPath.append("./");
	if (q.exists())
		searchPath.append(homedir);
	if (d.exists())
		searchPath.append(SCRIPT_DIR);

	if (q.exists())
		randomSearchPath.append(homedir);

	if (d.exists())
		randomSearchPath.append(SCRIPT_DIR);

}

fileManager::~fileManager()
{
}

QString fileManager::getCoordFile(QString file)
{
	QFileInfo base_or_script(file);

	if (isScript(base_or_script)) {
		std::cerr << "Script given on cmdline. Trying to create output...\n";

		QString coordFile = generateOutput(base_or_script);

		return coordFile;
	} else {
		std::cerr << "Output given on cmdline.\n";
		base_or_script.setFile(findFile(base_or_script));

		if (base_or_script.exists()) {
			return base_or_script.filePath();
		}
	}

	return QString::null;
}

QString fileManager::getRandomCoordFile(QString file)
{
	return findRandomFile(file);
}

bool fileManager::isScript(QFileInfo file)
{
	if (isBashScript(file) || isPerlScript(file)) {
		return true;
	}
	return false;
}

bool fileManager::isBashScript(QFileInfo file)
{
	if ((file.extension() == "sh")) {
		std::cerr << file.filePath() << " is a bash script\n";
		return true;
	}
	return false;
}

bool fileManager::isPerlScript(QFileInfo file)
{
	if (file.extension() == "pl") {
		std::cerr << file.filePath() << " is a perl script\n";
		return true;
	}
	return false;
}

QString fileManager::generateOutput(QFileInfo script)
{
	if (!script.exists()) {
		std::cerr << "File does not exist, we have to search\n";
		script.setFile(findFile(script));
	}

	if (!script.isFile())
		return QString::null;

	QString home = getenv("HOME");
	QDir destdir = script.dirPath();

	if (script.dirPath() == SCRIPT_DIR) {
		destdir.setPath(home + "/.qnotify/");
	}

	QString cmd;
	QString binary;

	if (script.extension() == "sh") {
		binary = "/bin/bash ";
	} else if (script.extension() == "pl") {
		binary = "/usr/bin/perl ";
	} else {
		std::cerr << "Unknown file-extension\n";
		return QString::null;
	}

	QDesktopWidget *d = QApplication::desktop();
	int w = d->width();	// returns desktop width
	int h = d->height();	// returns desktop height

	char wi[10], hi[10];
	sprintf(wi, "%d", w);
	sprintf(hi, "%d", h);

	QString width = wi;
	QString height = hi;

	cmd = binary + script.filePath();
	cmd.append(" " + width + " " + height + " " + ">");
	cmd.append(destdir.absPath() + "/" + script.baseName());

	cerr << "Command to be executed to generate output: " << cmd << endl;

	if (system(cmd.utf8()) < 0) {
		cerr << "Failed to generate output\n";
		return QString::null;
	}

	return destdir.absPath() + "/" + script.baseName();
}

QString fileManager::findFile(QFileInfo filename)
{
	for (QStringList::iterator it = searchPath.begin(); it != searchPath.end(); it++) {

		QString dir = *it;
		filename.setFile(QDir(dir), filename.fileName());
		std::cerr << "looking for " << filename.filePath() << std::endl;
		if (filename.exists()) {
			std::cerr << "found file " << filename.filePath() << " in " << dir
			    << std::endl;
			return filename.filePath();
		}
	}

	cerr << "Did not find " << filename.filePath() << " in searchpath\n";

	return QString::null;

}

QString fileManager::findRandomFile(QFileInfo file)
{
	QStringList fileList;

	std::cerr << "Trying to find random notification mode...\n";

	for (QStringList::iterator path_it = randomSearchPath.begin();
	     path_it != randomSearchPath.end(); path_it++) {

		QDir d = *path_it;
		d.setFilter(QDir::Files | QDir::NoSymLinks);

		const QFileInfoList *list = d.entryInfoList();
		QFileInfoListIterator it(*list);
		QFileInfo *fi;

		while ((fi = it.current()) != 0) {
//			std::cerr << "script: " << fi->filePath() << "\n";
			fileList.append(fi->filePath().utf8());
			++it;
		}

	}

	if (fileList.isEmpty()) {
		std::cerr << "No file found\n";
		return QString::null;
	}

	srand(time(NULL));	// Intialisierung der srand Funktion  //

	int ran = (int)rand() % (fileList.count());

	QString res = fileList[ran];

	std::cerr << "Choosing file: " << res << "\n";

	return res;
}
