/***************************************************************************
 *                                                                         *
 *                              QNotify                                    *
 *                                                                         *
 *            Copyright (C) 2006 Holger Macht <holger@homac.de>            *
 *                                                                         *
 *                 Author: Holger Macht <holger@homac.de>                  *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify it *
 * under the terms of the GNU General Public License as published by the   *
 * Free Software Foundation; either version 2 of the License, or (at you   *
 * option) any later version.                                              *
 *                                                                         *
 * This program is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
 * General Public License for more details.                                *
 *                                                                         *
 * You should have received a copy of the GNU General Public License along *
 * with this program; if not, write to the Free Software Foundation, Inc., *
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA                  *
 *                                                                         *
 ***************************************************************************/

#ifndef FILEMANAGER_H
#define FILEMANAGER_H

class QString;
class QFileInfo;

#include <qstringlist.h>

class fileManager {

      public:
	fileManager();
	~fileManager();

	// searches the searchpath for a script which basename is basename
	// and creates the appropriate output file. Returns true if succeeeded.
	// The function can be given a basename or a scriptname (with ending pl
	// or sh)
	QString getCoordFile(QString file);

	// takes a random output file from all scripts/coordFiles in searchpath
	// and returns it. Generates the coordFile if needed
	QString getRandomCoordFile(QString file);

      private:
	// holds all directories which define the search path, get initialised
	// in constructorx
	 QStringList searchPath;

	// holds all directories which define the search path for choosing
	// a random output file
	QStringList randomSearchPath;

	// checks the given string if it is a script or a basename
	bool isScript(QFileInfo file);

	// checks the given string if it is a bashscript
	bool isPerlScript(QFileInfo file);

	// checks the given string if it is a perlscript
	bool isBashScript(QFileInfo file);

	// generates the output from script and stores it in destdir. Returns the
	// newly created file or QString::null if generating failed.
	QString generateOutput(QFileInfo script);

	// returns the full path (with filename) where script is located or
	// QString::null
	QString findFile(QFileInfo script);

	// find s a random scipt/outputfile, generates the output in case
	// of a script, and returns the full path.      
	QString findRandomFile(QFileInfo file);

};

#endif
