/***************************************************************************
 *                                                                         *
 *                              QNotify                                    *
 *                                                                         *
 *            Copyright (C) 2006 Holger Macht <holger@homac.de>            *
 *                                                                         *
 *                 Author: Holger Macht <holger@homac.de>                  *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify it *
 * under the terms of the GNU General Public License as published by the   *
 * Free Software Foundation; either version 2 of the License, or (at you   *
 * option) any later version.                                              *
 *                                                                         *
 * This program is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
 * General Public License for more details.                                *
 *                                                                         *
 * You should have received a copy of the GNU General Public License along *
 * with this program; if not, write to the Free Software Foundation, Inc., *
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA                  *
 *                                                                         *
 ***************************************************************************/

//qt headers
#include <qapplication.h>
#include <qtimer.h>
#include <qtextcodec.h>

//own headers
#include "qnotify.h"
#include "settings.h"
#include "config.h"

//c++ headers
#include <iostream>

void usage();

globopts gopts;

int main(int argc, char *argv[])
{
	std::string str_pipe;

	//create the QApplication object
	QApplication *app = new QApplication(argc, argv);

	QStringList l;
	for (int i = 0; i < argc; i++) {
		l.append(argv[i]);
	}

	gopts.opts.append(l);
	QStringList::iterator it = l.find("-m");
	if (it != l.end()) {
		gopts.filename = *(++it);
	} else if (std::cin.rdbuf()->in_avail()) {
		//read everything from standard input
		while (std::cin.good()) {
			std::string str;
			std::cin >> str;
			str_pipe.append(str + " ");
		}
		// add the std input as message
		gopts.text.append(str_pipe);
	}

	//default codec is utf8
	QTextCodec::setCodecForTr(QTextCodec::codecForName("utf8"));

	//the base object
	QNotify *noty = new QNotify();

	QTimer::singleShot(0, noty, SLOT(notify()));

	return app->exec();
}

//usage
void usage()
{
	std::cerr << "\
\n\
QNotify " << VERSION << ", written by Holger Macht\n\
\n\
Usage:\n\
    qnotify <options>\n\
\n\
Options:\n\
    -m <text>          The message/text that should be displayed. If empty, standard input is used.\n\
    -o <fontsize>      The fontsize\n\
    -x <x-Postion>     The x-position of the window from the top left corner\n\
    -y <y-Postion>     The y-position of the window from the top left corner\n\
    -w <width>         The width of the window\n\
    -e <height>        The height of the window\n\
    -c <count>         How much time one single widget should be shown and hidden\n\
    -s <time>          The time how long the widget is shown untile the next hide\n\
    -t <time>          The time how long the widget is hidden until the next show\n\
    -n <color>         The foreground color/ color of the text\n\
    -z <color>         The background color\n\
    --fullscreen       Fullscreen mode\n\
    --alert            Alert mode. Give it a try\n\
    -a <character>     How the widgets should be aligned on the desktop. <character>\n\
    --no-shadow        Draw no a shadow\n\
                       is one of h,v, or d\n\
    --rounding-off     Do not draw rounded edges\n\
    --plain            A plain widget, no colors and shadow\n\
    --transparent      Show the window transparent\n\
    -l <filename>      The socketfile on which the daemon listens\n\
    --frame            Make a frame around the widget (dows not work with KDE yet\n\
    -T <space>         The space between the text/widgets and the whole widget\n\
    -R                 Rounding factor. from 1 to 20000\n\
    --center           Center the widget on the desktop\n\
    -F                 The file from which coordinates are taken\n\
    --message-box      Show a messagebox with a text and one ok button\n\
    --progress-dialog  Make the widget a progress dialog\n\
    --bar-background-color  The color of the progress bar\n\
    -f <filename>      The fifo-filename. Used for the progressbar\n\
    -u <step>          How many steps should be performed until 100 percent are reached in the\n\
                       progress bar\n\
    -v <time>          How long should it last until the progress bar is at 100 percent\n\
    --auto             Show random notifications\n\
    --no-alignment     Ignore already present widgets\n\
    -I <interval>      The interval how often a widget sequence moves on. The greater the value the\n\
                       slower the window will move\n\
    -h --help          Show this help screen\n\
\n\
For a more detailed description try 'man qnotify'\n\
\n\
There is also a daemon which keeps track of the shown windows and alignes them correctly.\n\
For bugreports or feature requests write to <holger@homac.de>\n\
\n\
";

}
