/***************************************************************************
 *                                                                         *
 *                              QNotify                                    *
 *                                                                         *
 *            Copyright (C) 2006 Holger Macht <holger@homac.de>            *
 *                                                                         *
 *                 Author: Holger Macht <holger@homac.de>                  *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify it *
 * under the terms of the GNU General Public License as published by the   *
 * Free Software Foundation; either version 2 of the License, or (at you   *
 * option) any later version.                                              *
 *                                                                         *
 * This program is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
 * General Public License for more details.                                *
 *                                                                         *
 * You should have received a copy of the GNU General Public License along *
 * with this program; if not, write to the Free Software Foundation, Inc., *
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA                  *
 *                                                                         *
 ***************************************************************************/

#include <qapplication.h>
#include <qfile.h>
#include <stdlib.h>

#include "qnotify.h"
#include "widgetNode.h"
#include "fileIO.h"
#include "settings.h"
#include "fileManager.h"
#include "barObj.h"
#include <qpushbutton.h>
#include <qmessagebox.h>

#include <iostream>

extern void usage();

extern globopts gopts;

QNotify::QNotify()
{
	page = 0;
	config = NULL;
	wid = NULL;

	readText();
}

QNotify::~QNotify()
{
}
#include <sys/time.h>
unsigned long QNotify::getTime()
{
	unsigned long start;
	timeval *tv = new(timeval);
	struct timezone *tz = new(struct timezone);
	gettimeofday(tv, tz);
	start = tv->tv_sec * 1000 + tv->tv_usec / 1000;
	delete tv;
	delete tz;
	tv = NULL;
	tz = NULL;

	return start;
}

void QNotify::notify()
{
	QStringList sum;
	QNOTIFY_POSITION widget_position = QNOTIFY_POS_BOTTOM_LEFT;

	sum = gopts.opts[page + 1] + gopts.opts[0];

	if (config != NULL) {
		delete config;
		config = NULL;
	}
	config = new Settings(sum);

	if (config->parse()) {
		config->setDefaults();
	} else {
		usage();
		exit(EXIT_FAILURE);
	}


	adjustSettings();


	wNode *lastLab;
	wNode *firstLab;
	int i;

	wid = new QNotifyPopup(0);

	barObj *bar = new barObj(wid);

 	wid->setBackgroundColor(gopts.backgroundColor);
	wid->setTextColor(gopts.foregroundColor);
	wid->setMargin(gopts.spacing);
	wid->addTextWidget(gopts.text[page], gopts.fontsize, Qt::AlignHCenter);
	wid->setTimesShown(gopts.count);
	wid->setRounding(gopts.rounding_factor);

	wid->setShowInterval((int)(gopts.showtime * 1000));
	wid->setHideInterval((int)(gopts.hidetime * 1000));

//	wid->render(gopts.width, gopts.height);

	if (gopts.center)
		widget_position = QNOTIFY_POS_MID_MID;

	if (gopts.pdialog) {
		bar->init();
		wid->addProgressBar(gopts.pbarHeight, gopts.pbarColor);
		wid->setShowInterval(-1);

		QPushButton *b = new QPushButton("Cancel", wid);
		wid->addWidget(b, Qt::AlignHCenter);
		connect(b, SIGNAL(clicked()), wid, SLOT(exitFalse()));

		wid->setShowInterval(-1);
		widget_position = QNOTIFY_POS_MID_MID;
	}
	
	if (gopts.msgbox) {
		QPushButton *b = new QPushButton("Ok", wid);
		wid->addWidget(b, Qt::AlignRight);
	}


	if (gopts.noshadow)
		wid->disableShadow();

	if (gopts.transparent) {
		wid->setTransparent();
	}

	if (gopts.coordFile != NULL || gopts.auto_widget) {
		fileManager *fM = new fileManager();

		if (gopts.auto_widget)
			gopts.coordFile = fM->getRandomCoordFile(gopts.coordFile);

		gopts.coordFile = fM->getCoordFile(gopts.coordFile);
		if (gopts.coordFile) {
			parseCoordFile(gopts.coordFile);
			wid->setSliding(xPositions, yPositions, intervals);
		} else {
			QMessageBox::warning(this,
					     "QNotify error",
					     "Unable to find requested position "
					     "file or script", 0, 0, 0);
			qApp->quit();
		}
	}


	if (gopts.no_alignment || !_widgetBag.init(gopts.socketfile)) {
		firstLab = lastLab = NULL;
		_bag_avail = false;
	} else
		_bag_avail = true;

	if (_bag_avail) {
		firstLab = new(wNode);
		i = _widgetBag.getFirst(firstLab);
		if (!i) {
			firstLab = lastLab = NULL;
		}

		if (firstLab) {
			lastLab = new(wNode);
			i = _widgetBag.getLast(lastLab);
			if (!i) {
				lastLab = NULL;
			}
		}
	}

	if (!firstLab && gopts.xPos < 0 && gopts.yPos < 0) {
		wid->setPosition(widget_position);
		wid->render(gopts.width, gopts.height);
	}
	else if ((firstLab) && !gopts.fullscreen) {
		wid->render(gopts.width, gopts.height);
		if (gopts.alignfunc == "h" || gopts.alignfunc == "horizontal")
			wid = alignfuncs.ltop_rbottom_h(wid, firstLab, lastLab);
		else if (gopts.alignfunc == "v" || gopts.alignfunc == "vertical")
			wid = alignfuncs.ltop_rbottom_v(wid, firstLab, lastLab);
		else if (gopts.alignfunc == "d" || gopts.alignfunc == "diagonal")
			wid = alignfuncs.ltop_rbottom_d(wid, firstLab, lastLab);
	}
	else {
		wid->render(gopts.width, gopts.height);
		wid->move(gopts.xPos, gopts.yPos);
	}

	wid->popup();

	if (_bag_avail) {
		wNode *n = new(wNode);
		n->x = wid->x();
		n->y = wid->y();
		n->width = wid->width();
		n->height = wid->height();
		n->id = 0;
		n->time = getTime() + wid->timeout()*1000;

		i = _widgetBag.writeFile(n);

		if (!i)
			std::cerr << "Error while writing the shown widget to the file\n";
		else
			id = n->id;
		delete n;
		_widgetBag.close();
	}

	if (!gopts.pdialog) {
		connect(wid, SIGNAL(popupFinished()), this, SLOT(next()));
	}
}

void QNotify::readText()
{
	QString text;

	QString filename = gopts.filename;

	if (filename == NULL) {
		return;
	}

	QFile file(filename);
//      QFile file( qApp->argv()[2] );

	QStringList widgets;

	if (file.open(IO_ReadOnly)) {
		gopts.text.clear();
		QTextStream stream(&file);
		QString line;

		line = stream.read();
		widgets = QStringList::split("<widget", line);
		file.close();
	}

	int pos;
	for (QStringList::iterator it = widgets.begin(); it != widgets.end(); it++) {
		QString h = *it;
		QString opts = h.section('>', 0, 0);
		QStringList l = QStringList::split(' ', opts, false);

		gopts.opts.append(l);

		QString text = h.section('>', 1, -1);
		gopts.text.append(text);
	}

}

bool QNotify::adjustSettings()
{

	if (gopts.text.count() == 0) {
		if (gopts.pdialog)
			gopts.text.append("Please wait...");
		else
			gopts.text.append("Notification");
	}

	if (gopts.alignmentStr == "right") {
		gopts.alignmentStr = "<p align=right>";
	} else if (gopts.alignmentStr == "left") {
		gopts.alignmentStr = "<p align=left>";
	} else {
		gopts.alignmentStr = "<p align=center>";
	}

	if (!config->isSet("time-show")) {
//		gopts.showtime = (float)gopts.text[page].length() * 0.05 + 2.0;
	}

	if (gopts.alert) {
		if (gopts.text[0] == "Notification")
			gopts.text[0] = "Alert!";

		gopts.foregroundColor = "black";
		gopts.backgroundColor = "red";
		gopts.count = 10;
		gopts.showtime = 0.5;
		gopts.hidetime = 0.5;
		gopts.fontsize = 50;

		gopts.fullscreen = true;
	}

	if (gopts.fullscreen) {
		gopts.xPos = 0;
		gopts.yPos = 0;
		gopts.rounding_factor = 0;
		gopts.width = qApp->desktop()->width();
		gopts.height = qApp->desktop()->height();
	}

	if (gopts.plain) {
		gopts.rounding_factor = 0;
		gopts.noshadow = true;
		gopts.foregroundColor = foregroundColor().name();
		gopts.backgroundColor = backgroundColor().name();
	}

	if (gopts.norounding) {
		gopts.rounding_factor = 0;
	}

	if (gopts.frame) {
		if (getWFlags() != Qt::WType_TopLevel) {
			std::cerr << "asjdasd\n";
//                      setWFlags(Qt::WType_TopLevel);
			reparent(0, Qt::WType_TopLevel, QPoint(0, 0), false);
		}
	} else {
		int flags = Qt::WType_TopLevel | Qt::WNoAutoErase |
		    Qt::WStyle_Customize | Qt::WX11BypassWM | Qt::WStyle_StaysOnTop;

		if (getWFlags() != flags)
			reparent(0, flags, QPoint(0, 0), false);
	}

	return true;
}

int QNotify::parseCoordFile(QString coordFile)
{
	QFile file(coordFile);

	if (file.open(IO_ReadOnly)) {
		QTextStream stream(&file);
		QString line;
		int i = 1;
		while (!stream.atEnd()) {
			line = stream.readLine();	// line of text excluding '\n'

			QString xpos = line.section(',', 0, 0);
			QString ypos = line.section(',', 1, 1);
			QString ival = line.section(',', 2, 2);

			if (xpos.isEmpty()) {
				printf("Error reading x position from file, assuming 0\n");
				xPositions.append(0);
			} else {
				xPositions.append(xpos.toInt());
			}

			if (ypos.isEmpty()) {
				printf("Error reading y position from file, assuming 0\n");
				yPositions.append(0);
			} else {
				yPositions.append(ypos.toInt());
			}

			if (ival.isEmpty()) {
				printf("Error reading time from file, assuming 1000\n");
				intervals.append(1000);
			} else {
				intervals.append(ival.toInt());
			}

			// printf("xpos: %s, ypos: %s, ival: %s\n", xpos.ascii(),
			//           ypos.ascii(), ival.ascii());
		}
		file.close();
		return 1;
	}

	return 0;
}

void QNotify::next()
{
	delete(wid);
	wid = NULL;

	page++;

	if (gopts.opts.size() ==  page + 1 || gopts.opts.size() == 1)
		qApp->quit();
	else 
		notify();
}
