/***************************************************************************
 *                                                                         *
 *                              QNotify                                    *
 *                                                                         *
 *            Copyright (C) 2006 Holger Macht <holger@homac.de>            *
 *                                                                         *
 *                 Author: Holger Macht <holger@homac.de>                  *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify it *
 * under the terms of the GNU General Public License as published by the   *
 * Free Software Foundation; either version 2 of the License, or (at you   *
 * option) any later version.                                              *
 *                                                                         *
 * This program is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
 * General Public License for more details.                                *
 *                                                                         *
 * You should have received a copy of the GNU General Public License along *
 * with this program; if not, write to the Free Software Foundation, Inc., *
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA                  *
 *                                                                         *
 ***************************************************************************/

#ifndef QNOTIFY_H
#define QNOTIFY_H

#include "align.h"
#include "fileIO.h"
#include "settings.h"
#include "qnotify_popup.h"

class QStringList;
class QSocketNotifier;
class QTimer;

/** @brief central object for displaying a notification */
class QNotify : public QWidget {
      Q_OBJECT

public:
      /** @brief constructor */
      QNotify();

      /** @brief destructor */
      ~QNotify();

      static unsigned long getTime();
private:
      /** @brief reads the texts and opts from a widget file */
      void readText();



      
      /** @brief parses a configuration file */
      int parseCoordFile(QString dir = "./");

      /** @brief list for sliding */
      QValueList < int >xPositions, yPositions, intervals;

      /** @brief adjusts settings to need */
      bool adjustSettings();

      /** @brief the current widget displated (if given a <widget>
       * file) */
      unsigned int page;

      /** @brief object for parsing configurations */
      Settings *config;
      
      /** @brief if qnotifyd is running */
      bool _bag_avail;

      /** @brief how the popup should be aligned */
      align alignfuncs;
      
      /** @brief object for communication with qnotifyd */
      FileIO _widgetBag;

      /** @brief the popup */
      QNotifyPopup *wid;

      /** @brief id of the popup/widget */
      int id;

private slots:
      /** @brief popup a notification */
      void notify();

      /** @brief move on to the next <widget> */
      void next();
};

#endif // QNOTIFY_H
