/***************************************************************************
 *                                                                         *
 *                              QNotify                                    *
 *                                                                         *
 *            Copyright (C) 2006 Holger Macht <holger@homac.de>            *
 *                                                                         *
 *                 Author: Holger Macht <holger@homac.de>                  *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify it *
 * under the terms of the GNU General Public License as published by the   *
 * Free Software Foundation; either version 2 of the License, or (at you   *
 * option) any later version.                                              *
 *                                                                         *
 * This program is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
 * General Public License for more details.                                *
 *                                                                         *
 * You should have received a copy of the GNU General Public License along *
 * with this program; if not, write to the Free Software Foundation, Inc., *
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA                  *
 *                                                                         *
 ***************************************************************************/

#include "settings.h"

#include <iostream>

#include <stdlib.h>

extern globopts gopts;

Settings::Settings(const QStringList & a)
	: GetOpt(a)
{
	setParams();
}

Settings::Settings(int argc, char *argv[])
:GetOpt(argc, argv)
{

	setParams();
}

Settings::~Settings()
{
}

void Settings::setParams()
{
	addOption('m', "message", &text);

	QString home = getenv("HOME");
	QString sf = home + "/qnotify.socket";

	addOption('l', "socketfile", &gopts.socketfile);

	QString ff = home + "/qnotify.fifo";
	addOption('f', "fifo-filename", &gopts.fifofile);

	addOption('a', "alignment-function", &gopts.alignfunc);

	addOption('w', "width", &width);
	addOption('e', "height", &height);
	addOption('x', "x-pos", &xPos);
	addOption('y', "y-pos", &yPos);
	addOption('o', "fontsize", &fontsize);

	addOption('c', "count", &count);
	addOption('s', "time-show", &showtime);
	addOption('t', "time-hide", &hidetime);
	addOption('n', "foreground-color", &gopts.foregroundColor);
	addOption('z', "background-color", &gopts.backgroundColor);
	addOption('u', "step", &step);
	addOption('v', "fulltime", &fulltime);
	addOption('T', "spacing", &spacing);
	addOption('R', "rounding-factor", &rounding);
	addOption('N', "bar-background-color", &gopts.pbarColor);
	addOption('H', "bar-height", &pbarHeight);

	addOption('F', "coord-file", &gopts.coordFile);

//      addSwitch( "help", &gopts.help );
	addSwitch("rounding-off", &gopts.norounding);
	addSwitch("fullscreen", &gopts.fullscreen);
	addSwitch("alert", &gopts.alert);
	addSwitch("progress-dialog", &gopts.pdialog);
	addSwitch("plain", &gopts.plain);
	addSwitch("no-shadow", &gopts.noshadow);
	addSwitch("frame", &gopts.frame);
	addSwitch("center", &gopts.center);
	addSwitch("transparent", &gopts.transparent);
	addSwitch("message-box", &gopts.msgbox);
	addSwitch("auto", &gopts.auto_widget);
	addSwitch("no-alignment", &gopts.no_alignment);
}

void Settings::setDefaults()
{
	gopts.width = width.toInt();
	gopts.height = height.toInt();
	gopts.xPos = xPos.toInt();
	gopts.yPos = yPos.toInt();
	gopts.fontsize = fontsize.toInt();
	gopts.count = count.toInt();
	gopts.showtime = showtime.toFloat();
	gopts.hidetime = hidetime.toFloat();
	gopts.step = step.toInt();
	gopts.fulltime = fulltime.toInt();
	gopts.pbarHeight = pbarHeight.toInt();
	gopts.spacing = spacing.toInt();
	gopts.rounding_factor = rounding.toInt();

	if (gopts.text.count() == 0) {
		gopts.text.append(text);
	}

	if (gopts.text[0] == NULL) {
		gopts.text[0] = "Notification";
	}

	if (gopts.alignfunc == NULL)
		gopts.alignfunc = "h";

	if (gopts.fifofile == NULL) {
		char *home = getenv("HOME");
		gopts.fifofile = home;
		gopts.fifofile.append("/qnotify.fifo");;
	}

	if (gopts.socketfile == NULL) {
		char *home = getenv("HOME");
		gopts.socketfile = home;
		gopts.socketfile.append("/qnotify.widgets");;
	}

	if (gopts.pdialog || gopts.msgbox) {
		gopts.center = true;
	}

	if (width == NULL)
		gopts.width = -1;
	if (height == NULL)
		gopts.height = -1;
	if (xPos == NULL)
		gopts.xPos = -1;
	if (yPos == NULL)
		gopts.yPos = -1;
	if (fontsize == NULL)
		gopts.fontsize = 12;
	if (count == NULL)
		gopts.count = 1;
	if (showtime == NULL)
		gopts.showtime = -1;
	if (hidetime == NULL)
		gopts.hidetime = 1;
	if (gopts.foregroundColor == NULL)
		gopts.foregroundColor = "white";
	if (gopts.backgroundColor == NULL)
		gopts.backgroundColor = "#3b508a";
	if (gopts.pbarColor == NULL)
		gopts.pbarColor = "#3b508a";
	if (step == NULL)
		gopts.step = 1;
	if (fulltime == NULL)
		gopts.fulltime = 5;
	if (pbarHeight == NULL)
		gopts.pbarHeight = -1;
	if (rounding == NULL)
		gopts.rounding_factor = 3000;
	if (spacing == NULL)
		gopts.spacing = 10;

}

void Settings::clear()
{
	gopts.norounding = false;
	gopts.fullscreen = false;
	gopts.alert = false;
	gopts.pdialog = false;
	gopts.plain = false;
	gopts.noshadow = false;
	gopts.frame = false;
	gopts.spacing = 10;
	gopts.center = false;
	gopts.rounding_factor = 3000;
	gopts.transparent = false;
	gopts.auto_widget = false;
	gopts.no_alignment = false;

//      gopts.text.clear();
//      gopts.opts.clear();
	gopts.alignfunc = '0';
	gopts.fifofile = "";
	gopts.socketfile = "";
	gopts.width = 0;
	gopts.height = 0;
	gopts.xPos = 0;
	gopts.yPos = 0;
	gopts.fontsize = 0;
	gopts.count = 0;
	gopts.showtime = 0;
	gopts.hidetime = 0;
	gopts.foregroundColor = "";
	gopts.backgroundColor = "";
	gopts.pbarColor = "";
	gopts.step = 0;
	gopts.fulltime = 0;

}
